#include "dictionary.h"
#include <QFile>
#include <QMessageBox>
#include <QDebug>
#include <QHashIterator>
Dictionary::Dictionary(QObject *parent) :
    QObject(parent)
{
    this->menuBarCount=0;
}

//----------------
void Dictionary::loadData(QString fileName){

    if (fileName.isEmpty())
        exit(1);

    QFile file(fileName);
    if (!file.open(QFile::ReadOnly | QFile::Text)) {
        QMessageBox::warning(0, tr("DOM Bookmarks"),
                             tr("Cannot read file %1:\n%2.")
                             .arg(fileName)
                             .arg(file.errorString()));
        exit(1);
    }


    this->m_data = Parser::load(&file);
}


void Dictionary::process(){
    QDomElement root =  m_data->firstChildElement("root");
    if(root.isNull()){
        qDebug()<<"there is no ROOT element";
        exit(1);
    }

    //qDebug()<<"root OK";
    QDomElement menubar = root.firstChildElement("menubar");
    int applicationsCount =0;
    while(!menubar.isNull()){
        applicationsCount++;
        processMenuBar(menubar);
        menubar = menubar.nextSiblingElement("menubar");
    }
    qDebug()<< applicationsCount <<"applications tested";
}

QString nameItem(QDomElement item){
    QString name = item.attribute("title");
    if(name.isEmpty())
        return "";
    if(name.at(name.size()-1).unicode()==8230){
        name = name.mid(0,name.size()-1);
    }else if(name.endsWith("...")){
        name = name.mid(0, name.size()-3);
    }

    return name;
}

void Dictionary::processMenuBar(QDomElement n){
    this->resetMenubarData();
    this->menuBarCount++;
    QString appName = n.attribute("appname");
    QStringList titles; titles << appName;
    //qDebug()<<"menubar: "<<appName;

    QDomElement menu = n.firstChildElement("menu");
    while(!menu.isNull()){
        processMenu(menu, titles);
        menu = menu.nextSiblingElement("menu");
    }

    postProcessMenuBar();
    this->insertNovelItems();
    this->insertNovelAssociations();
//    QHashIterator< QPair<QString,QString>, float> it(this->menubarAssociations);
//    while(it.hasNext()){
//        it.next();
//        QPair<QString, QString> key = it.key();
//        if(it.value()>0)
//        qDebug()<< key.first << "\t"<<key.second << "\t"<< it.value() << "\t"<<menubarInfo[key.first].listTitles << "\t"<< menubarInfo[key.second].listTitles;
//    }
}

//File
//File>export>pdf>
//File>export>png>

//File>recent>
//File
//1
//dist<1,1>

//File>recent>
//File>export>png>
//1 => 2-1=1 + 3-1=2
//dist<sim:1; diss:3>

float Dictionary::associationScore(SingleItem si1, SingleItem si2){
    QStringList l1 = si1.listTitles; l1.removeAt(0);
    QStringList l2 = si2.listTitles; l2.removeAt(0);

    int curIndex=0;
    for(int i=0; i<100;i++){
        if(i <l1.size() && i <l2.size()){
            if(l1[i] != l2[i]){//same menu
                curIndex =i;
                break;
            }
        }else{
            curIndex = i;
            break;
        }
    }

    if(curIndex==0)
        return 0;

    float similarity = curIndex;
    float dissimilarity = l1.size() - curIndex + l2.size() - curIndex;
    if(dissimilarity ==0){
        qDebug()<<"wir haben ein problem "<<l1 <<" "<<l2 <<curIndex;
    }

    return 1./(1. + dissimilarity);
}

//----------------------
void Dictionary::postProcessMenuBar(){
    QHashIterator<QString,SingleItem> it(this->menubarInfo);
    QVector< SingleItem> items;
    while(it.hasNext()){
        it.next();
        QString str = it.key();
        SingleItem item = it.value();
        items.push_back(item);
    }

    for(int i=0; i< items.size();i++){
        for(int j=i+1; j<items.size();j++){
            if(menubarAssociations.contains(QPair<QString, QString>(items[i].name, items[j].name)) ||
                    menubarAssociations.contains( QPair<QString, QString>(items[j].name, items[i].name) ) ){
                //existing associations
            }else{
                float score = associationScore(items[i], items[j]);
                //if(score>0){
                    menubarAssociations.insert( QPair<QString, QString>(items[i].name, items[j].name) , 0.33*score);
                //}
            }

        }
    }

}


void Dictionary::processMenu(QDomElement menu, QStringList _titles){
    QString title = menu.attribute("title");
    QStringList titles = _titles; titles << title;
    if(titles.at(0)==title){
        return;
    }
    //qDebug()<<"   -Menu: "<<title;

    QDomElement item = menu.firstChildElement();
    int idSeparator=0;
    QVector<QDomElement> prevItems;
    QVector<int> prevLogicalGroups;
    QVector<int> prevPos;
    int pos=0;
    while(!item.isNull()){
        if(item.tagName()=="menuitem"){
            if(nameItem(item).contains(".")){
                //qDebug()<<">>>>>>>>>>>>>item "<<nameItem(item)<< " contains .";
            }else{
                //get infos about the item

                bool toInsert = processItem(item, pos,titles);
                if(toInsert){
                    //create associations with previous items in the menu
                    for(int i=0; i< prevItems.size();i++){
                        QString prevName = nameItem(prevItems[i]);//prevItems[i].attribute("title");
                        QString curName = nameItem(item);//item.attribute("title");
                        if(prevName!=curName){

                            QPair<QString, QString> ass1(prevName, curName);
                            QPair<QString, QString> ass2(prevName, curName);
                            if(menubarAssociations.contains(ass1) || menubarAssociations.contains(ass2) ){
                                qDebug()<<"error this association already exists...."<<prevName << " "<<curName;
                            }

                            float score =0.33;
                            if(idSeparator == prevLogicalGroups[i]){
                                score += 0.33;
                            }
                            float d = pos - prevPos[i];
                            score += 0.33/d;
                            menubarAssociations.insert(ass1,score);
                        }
                    }
                    prevItems.push_back(item);
                    prevLogicalGroups.push_back(idSeparator);
                    prevPos.push_back(pos);
                }
            }
            pos++;
        }else if(item.tagName()=="separator"){
            idSeparator++;

        }else if(item.tagName()=="menu"){
            if(nameItem(item)!= "Open Recent"){
                this->processMenu(item, titles);
            }

            pos++;
        }
        item = item.nextSiblingElement();
    }
}


//Association assInfo;
//QPair<QString,QString> winnerPair;
//if(associations.contains(ass1)){
//    assInfo = associations[ass1];
//    winnerPair = ass1;
//}else if(associations.contains(ass2)){
//    assInfo = associations[ass2];
//    winnerPair = ass2;
//}else{
//    associations.insert(ass1,assInfo);
//    winnerPair = ass1;
//}


//assInfo.value.push_back(score);
//associations[winnerPair] = assInfo;


//used to the update at the end of menubar
void Item::insert(SingleItem si){
    //pos
    if(pos.contains(si.pos)){
        pos[si.pos] = pos[si.pos]+1;
    }else{
        pos.insert(si.pos,1);
    }

    //titles
    QString title = si.listTitles.at(si.listTitles.size()-1);
    if(titles.contains(title)){
        titles[title] = titles[title]+1;
    }else{
        titles.insert(title, 1);
    }

    //hotkey

}


bool Dictionary::processItem(QDomElement item, int pos, QStringList titles){
    QString name = nameItem(item);
    SingleItem data;
    if(!menubarInfo.contains(name)){
        data.name = name;
        data.pos = pos;
        data.listTitles = titles;
        menubarInfo.insert(name, data);
    }else{
        qDebug()<<name << " already exists in the menubar : "<<titles <<" "<<menubarInfo[name].listTitles;
        return false;
    }
    return true;
}


QString Item::toString(){
    //    QMapIterator<int,int> it(this->pos);
    //    QString res;
    //    while(it.hasNext()){
    //        it.next();
    //        res += QString::number(it.key()) + "["+ QString::number(it.value()) + "] ";
    //    }

    QMapIterator<QString,int> it2(this->titles);
    QString res2;
    while(it2.hasNext()){
        it2.next();
        res2 += it2.key() + "["+ QString::number(it2.value()) + "] ";
    }

    return res2;
}

QString Association::toString(){
    QString res;
    for(int i=0;i<value.size();i++){
        res+= QString::number(value[i]);
    }
    return res;
}

void Dictionary::printItems(bool verbose, bool toFile){
    QHashIterator<QString, Item> it(info);
    QString res ="";
    while(it.hasNext()){
        it.next();
        Item item = it.value();
        res +=it.key();
        QMapIterator<QString,int> it2(item.titles);
        while(it2.hasNext()){
            it2.next();
            QString title = it2.key();
            int count = it2.value();
            for(int i=0;i<count;i++){
                res+=","+ title;
            }
        }
        res+="\n";
    }

    if(toFile){
        QFile file("/Users/gbailly/userBehavior/titles.txt");
        if (!file.open(QIODevice::WriteOnly | QIODevice::Text))
            return;

        QTextStream out(&file);
        out << res;
    }

    qDebug()<<res;

}

void Dictionary::printAssocations(bool verbose, bool toFile){
    QHashIterator< QPair<QString,QString>, Association> it(associations);
    QString res="";
    while(it.hasNext()){
        it.next();
        QPair<QString,QString> pair = it.key();
        Association ass = it.value();

        float avg=0;
        for(int i=0;i<ass.value.size();i++){
            if(verbose){
                //res+= QString::number(ass.value[i]) + " ";
            }
            avg += ass.value[i];
        }
        if(ass.value.size()!=0){
            avg = avg / ass.value.size();
        }
        //res += pair.
        //res += avg;
        if(avg>0){
            res += pair.first + "," + pair.second + "," +  QString::number(avg) + "," + QString::number((float)ass.value.size() / (float)this->menuBarCount) + "\n";
        }
    }

    if(toFile){
        QFile file("/Users/gbailly/userBehavior/associations.txt");
        if (!file.open(QIODevice::WriteOnly | QIODevice::Text))
            return;

        QTextStream out(&file);
        out << res;
    }

    qDebug()<<res;
}



