#ifndef DICTIONARY_H
#define DICTIONARY_H

#include <QObject>
#include "parser.h"
#include <QHash>
#include <QStringList>
#include <QVector>
#include <QKeySequence>
#include <QStringList>
#include <QMap>
#include <QDebug>

class SingleItem{
public:
  QString name;
  int pos;
  QStringList listTitles;
  void toString(){ qDebug()<< name<< " " << pos << " "<< listTitles; }
};

class Item{
public:

  QMap<QKeySequence, int> hotkeys;
  QMap<QString, int> titles;
  //QMap<int, QMap<QString, int> > titles; //key depth ; < key=title; value number of occurences>
  QMap<int, int> pos; //key: pos; int number of occurences

  void insert(SingleItem si);

  QString toString();

};

class Association{
public:
    QVector<float> value;
    QString toString();
    void insertAssociation(float v){
        value.push_back(v);
    }
};

class Dictionary : public QObject
{
    Q_OBJECT
public:
    explicit Dictionary(QObject *parent = 0);

    void loadData(QString fileName);
    void process();
    void processMenuBar(QDomElement);
    void processMenu(QDomElement, QStringList titles);
    bool processItem(QDomElement, int pos, QStringList titles);

    void printItems(bool verbose=false, bool toFile=false);
    void printAssocations(bool verbose=false, bool toFile=false);
    void postProcessMenuBar();
    float associationScore(SingleItem si1, SingleItem si2);

    void resetMenubarData(){
        menubarAssociations.clear();
        menubarInfo.clear();
    }

    void insertNovelItems(){
        QHashIterator<QString, SingleItem> it(menubarInfo);
        while(it.hasNext()){
            it.next();
            SingleItem itemInfo = it.value();
            QString itemName = it.key();

            if(info.contains(itemName)){
                Item globalItem = info[itemName];
                globalItem.insert(itemInfo);
                info[itemName] = globalItem;

            }else{
                Item data; data.insert(itemInfo);
                info.insert(itemName, data);
            }

        }
    }

    void insertNovelAssociations(){
        QHashIterator< QPair<QString, QString>, float> it(menubarAssociations);
        while(it.hasNext()){
            it.next();
            float score = it.value();
            QPair<QString, QString> key = it.key();

            if(associations.contains(key)){
                Association globalAss = associations[key];
                globalAss.insertAssociation(score);
                associations[key] = globalAss;

            }else{
                Association data; data.insertAssociation(score);
                associations.insert(key, data);
            }

        }
    }

    QHash<QString, Item> info;
    QHash< QPair<QString,QString>, Association> associations;

    QHash< QPair<QString,QString>, float> menubarAssociations;
    QHash<QString, SingleItem> menubarInfo;

    Parser* m_data;
    int menuBarCount;
};

#endif // DICTIONARY_H
